<?php

namespace App\Service;

use App\Exception;

class IntegrationService {

    private $util;
    private $consumerService;
    private $productService;
    private $paymentFactory;
    private $consumerFactory;
    private $storeFactory;
    private $orderFactory;
    private $productFactory;
    private $branchFactory;
    private $integrationFactory;

    public function __construct (\App\Util\Util $util, 
                                 \App\Service\ConsumerService $consumerService,
                                 \App\Service\ProductService $productService, 
                                 \App\Factory\PaymentFactory $paymentFactory, 
                                 \App\Factory\ConsumerFactory $consumerFactory, 
                                 \App\Factory\StoreFactory $storeFactory,
                                 \App\Factory\OrderFactory $orderFactory, 
                                 \App\Factory\ProductFactory $productFactory,
                                 \App\Factory\BranchFactory $branchFactory,
                                 \App\Factory\IntegrationFactory $integrationFactory) {
        
        $this->util = $util;
        
        $this->consumerSerivce      = $consumerService;
        $this->productService       = $productService;
        
        $this->paymentFactory       = $paymentFactory;
        $this->consumerFactory      = $consumerFactory;
        $this->storeFactory         = $storeFactory;
        $this->orderFactory         = $orderFactory;
        $this->productFactory       = $productFactory;
        $this->branchFactory        = $branchFactory;
        $this->integrationFactory   = $integrationFactory;
    }
    
    public function generalParameters ($branch) {
        $vendor        = $this->storeFactory->getStandartVendor($branch);
        $store         = $this->storeFactory->getStandartStore($branch, $vendor);
        $cashier       = $this->storeFactory->getStandartCashier($branch, $vendor);
        $idControlProd = $this->storeFactory->getProductControlId($branch, $store);
        
        return array(
            "CDFILIAL"     => $branch,
            "CDVENDEDOR"   => $vendor,
            "CDLOJA"       => $store["CDLOJA"],
            "CDCAIXA"      => $cashier["CDCAIXA"],
            "CDOPERADOR"   => $cashier["CDOPERADOR"],
            "IDCONTROPROD" => $idControlProd
        );
    }

    public function validateCashierSinc($filial){
 	    
 	    $caixaSinc = $this->orderFactory->fetchIsCashierSinc($filial);
 	    if(!isset($caixaSinc) || empty($caixaSinc)){
 	        throw new Exception\ParamsException("Caixa sincronizador não encontrado na filial $filial, Habilite a flag de sincronização de caixa.", 404);
 	    }
 	}
    
    public function openOrderBill ($orderData, $generalParams, $consumerDetails) {
        $sale = $this->util->newCode("VENDAREST".$generalParams["CDFILIAL"], 10);
        $comd = $this->util->newCode("COMANDAVEN".$generalParams["CDFILIAL"], 10);
        
        if(empty($generalParams["CDOPERADOR"])) {
            throw new Exception\ValidationException("Não foi encontrado operador relacionado ao vendedor padrão (".$generalParams["CDVENDEDOR"].") da filial " . $generalParams["CDFILIAL"], 400);
        }
        
        $operator = $this->orderFactory->validateOperator($generalParams);
        if(empty($operator)) {
            throw new Exception\ValidationException("
                O operador {$generalParams['CDOPERADOR']} não está associado a filial {$generalParams['CDFILIAL']}.<br>
                Para atrelar o operador a esta filial, verifique a seção 'Problemas Frequentes', no manual da integração.");
        }
        
        $isDuplicated = $this->orderFactory->isDuplicated($generalParams, $orderData);
        if($isDuplicated) {
            throw new Exception\IntegrationException("O pedido {$orderData['id']} já foi integrado pelo sistema.");
        }
        
        $this->orderFactory->insertSale($sale, $generalParams, $consumerDetails);
        
        $orderData["NRVENDAREST"] = $sale;
        $orderData["NRCOMANDA"]   = $comd;
        $orderData["IDSTCOMANDA"] = $generalParams["IDCONTROPROD"] === "S" ? "1" : "3";

        $orderData["origin"] = 'DLV_NMO';

        /*
            $orderData["origin"] ->
                2 = Web Delivery      - DLV_NWB
                3 = App Android       - DLV_NAD
                4 = App iOS           - DLV_NOS
                5 = Central Delivery  - DLV_NMO
                6 = Chatbot Messenger - DLV_NBT
        */
        
        // switch($orderData["origin"]){
        //     case 2:
        //         $orderData["IDORGCMDVENDA"] = 'DLV_NWB';
        //         break;
        //     case 3:
        //         $orderData["IDORGCMDVENDA"] = 'DLV_NAD';
        //         break;
        //     case 4:
        //         $orderData["IDORGCMDVENDA"] = 'DLV_NOS';
        //         break;
        //     case 5:
        //         $orderData["IDORGCMDVENDA"] = 'DLV_NMO';
        //         break;
        //     case 6:
        //         $orderData["IDORGCMDVENDA"] = 'DLV_NBT';
        //         break;
        //     default:
        //         $orderData["IDORGCMDVENDA"] = 'DLV_NMO';
        // }
        
        $this->orderFactory->insertSaleCommand($orderData, $generalParams, $consumerDetails);
        
        $total = $orderData["total"];
        
        $orderData["NRSEQMOVDLV"] = $this->util->newCode("MOVCAIXADLV".$generalParams["CDFILIAL"].$sale, 10);
        $orderData["CDTIPORECE"]  = $this->paymentFactory->getReceiptType($orderData["payment_method"]);
        $orderData["total"]       = $this->orderFactory->checkForChange($orderData);
        $this->orderFactory->insertCashMoviment($orderData, $generalParams, $consumerDetails);
        
        if ($orderData["troco"] - $total > 0) {
            $orderData["total"] = $total;
            
            $movCaixa = $this->util->newCode("MOVCAIXADLV".$generalParams["CDFILIAL"].$sale, 10);
            $this->orderFactory->insertChange($orderData, $generalParams, $consumerDetails, $movCaixa);
        }
        
        $generalParams["NRSEQPRODCUP"] = $this->util->newCode("ITCOMANDAVEN".$generalParams["CDFILIAL"], 10);

        $products     = [];
        $nrSeqProdCom = 0;
        
        foreach ($orderData["ItemOrder"] as $item) {
            
            $processedProduct = $this->productService->processProduct($generalParams, $item, $orderData, $nrSeqProdCom);
            
            if (is_array($processedProduct) && !isset($processedProduct["CDPRODUTO"]))
                $products = array_merge($products, $processedProduct);
            else
                array_push($products, $processedProduct);
        }
        
        if(bccomp($orderData["total_discount"], "0.00", 2) == 1) {
            $products = $this->productService->shareDiscount($orderData["total_discount"], $products);
        }

        if(empty($products)) {
            throw new Exception\ValidationException("Falha na validação de itens do pedido {$orderData['id']}", 1515);
        }
        
        foreach ($products as $product) {

            if(empty($product['VRPRECCOMVEN']) || $product['VRPRECCOMVEN'] < 0) {
                throw new Exception\ValidationException("O produto ({$product['CDPRODUTO']}) possui preço inferior a 0.01 centavo. Valor: {$product['VRPRECCOMVEN']}", 191);
            }
            
            if(bccomp($product['VRDESCCOMVEN'], bcmul($product['VRPRECCOMVEN'], $product['QTPRODCOMVEN'], 2), 2) == 1) {
                throw new Exception\ValidationException("O produto ({$product['CDPRODUTO']}) possui DESCONTO superior ao VALOR: Desconto -> {$product['VRDESCCOMVEN']} Valor -> {$product['VRPRECCOMVEN']}", 192);
            }

            $this->productService->insertProduct($product, $generalParams, $orderData);
        }
        
        return [
            'CDFILIAL'      => $generalParams["CDFILIAL"],
            'NRCOMANDA'     => $orderData["NRCOMANDA"],
            'NRCOMANDAEXT'  => $orderData["order_number"],
            'NRVENDAREST'   => $orderData["NRVENDAREST"]
        ];
    }
    
    public function getConfiguration() {
        return $this->integrationFactory->getConfiguration();
    }
    
    public function disableIntegration() {
        $this->integrationFactory->disableIntegration();
    }
    
    public function validateOrder($cdFilial, $nrVendaRest) {
        $this->orderFactory->validateOrder($cdFilial, $nrVendaRest);
    }
}