<?php

namespace App\Service;

use App\Exception;

class ProductService {

    private $util;
    private $productFactory;
    
    public function __construct (\App\Util\Util $util, 
                                 \App\Factory\ProductFactory $productFactory) {
        $this->util    = $util;
        $this->productFactory = $productFactory;
    }
    
    public function processProduct($generalParams, $product, $order, &$nrSeqProdCom) {
        $observations = [];

        //Tratamento de produto com código externo e sem preço
        if (!empty($product["variacao_ref"]) && empty($product["price"]))
            throw new Exception\ParamsException("<p align='justify'>
                O produto '{$product['title']}' foi cadastrado sem preço e o código externo ({$product["variacao_ref"]}) está no formato de produto com preço. A obrigatoriedade do preço e do código externo dependem das regras abaixo:</p><br>
                <li>Caso o código externo esteja correto, será necessário atrelar um valor igual ou superior a R$ 0,01.</li><br>
                <li>Caso o produto seja um agrupador sem preço (Por exemplo, 'Monte o seu combo'), é necessário remover o código externo do produto.</li><br>
                <p align='justify'>Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");
            
        //Tratamento de produto sem código externo e com preço
        if (empty($product["variacao_ref"]) && !empty($product["price"]))
            throw new Exception\ParamsException("<p align='justify'>
                O produto '{$product['title']}' foi cadastrado com preço e sem código externo. A obrigatoriedade do preço e do código externo dependem das regras abaixo:</p><br>
                <li>Caso o preço do produto esteja correto, será necessário atrelar um código externo no formato 'xx.xx.xx.xx.xx'.</li><br>
                <li>Caso o produto seja um agrupador sem preço (Por exemplo, 'Monte o seu combo'), é necessário remover o preço do produto.</li><br>
                <p align='justify'>Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");

        $isCombo = false; // mesmo se houver objetos dentro de complementsCategories, se eles não tiverem preços, o item pai será considerado simples
        foreach ($product["ComplementCategories"] as $complementCategories) {
            foreach ($complementCategories["Complements"] as $item) {
                if(isset($item['price_un']) && $item['price_un']) {
                    $isCombo = true;
                }
            }
        }        

        /* ----- Depreciado
        // Se existir observações:
        if(isset($product["ComplementCategories"]) && !empty($product["ComplementCategories"])) {
        */    

        if($isCombo) {
            //Combo com código externo e com preço
            if(!empty($product["variacao_ref"]) && !empty($product["price"])) {
                $processedProduct = $this->processProductAsComboWithExtCode($generalParams, $product, $order, $nrSeqProdCom, $observations);
            }
            
            //Combo sem código externo e sem preço
            else {
                $processedProduct = $this->processProductAsComboWithoutExtCode($generalParams, $product, $order, $nrSeqProdCom);
            }
        }
        else {
            $processedProduct = $this->processProductAsSimple($generalParams, $product, $order, $nrSeqProdCom);
        }
        return $processedProduct;
    }
        
    private function processProductAsSimple ($generalParams, $product, $orderData, &$nrSeqProdCom) {

        if ($product["variacao_ref"] === "") {
            if ($product["ref"] === "")
                throw new Exception\ParamsException("
                    O produto '{$product['title']}' não possui código externo e ele é obrigatório. Verifique no Odhen Retail o código externo deste produto e faça a correção no portal da Neemo. Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
            
            $product_ref = $product["ref"];
        } else $product_ref = $product["variacao_ref"];
        
        $productDetails = $this->productFactory->getProduct($product_ref);
        
        if (empty($productDetails))
            throw new Exception\ParamsException("<p align='justify'>
                Não existe um produto atrelado ao código externo informado.</p><br>
                <li>Produto: {$product['title']}.</li><br>
                <li>Código externo: {$product['variacao_ref']}.</li><br>
                <p align='justify'>Verifique no Odhen Retail o código externo deste produto e faça a correção no portal Neemo.<br>
                Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.");
                
        $productDetails = array_merge($productDetails, $product);

        $productNrSeqProdCom = str_pad((string) ($nrSeqProdCom + 1), 6, '0', STR_PAD_LEFT);
        $nrSeqProdCom++;

        $productDetails["NRPRODCOMVEN"] = $productNrSeqProdCom;
        
        $productDetails["TXPRODCOMVEN"] = " ";
        $productDetails["TXPRODCOMVEN"] .= empty($productDetails["notes"]) ? "" : $productDetails["notes"]."; ";

        if (count($productDetails["ComplementCategories"]) > 0) {
            foreach ($productDetails["ComplementCategories"] as $categorie) {
                foreach ($categorie["Complements"] as $obs) {
                    if(isset($obs['ref']) && !empty($obs['ref']) && (strpos($obs['ref'], "@") !== false)) {
                        $obsTitle = $this->getObservationName($obs, $generalParams);

                        for ($i = 0 ; $i < $obs["quantity"] ; $i ++)
                            $productDetails["TXPRODCOMVEN"] .= $obsTitle."; ";
                        
                    }else if(isset($obs['price_un']) && !$obs['price_un'] && !empty($obs['title'])) {
                        // Observações tipo O e R
                        
                        if(!isset($obs['ref']) || empty($obs['ref']))
                            throw new Exception\ParamsException("<p align='justify'>
                                O subitem {$obs['title']} foi cadastrado sem preço e sem código externo. Apesar do preço não ser uma exigência (ver abaixo), o código externo é obrigatório para todos os subitems.</p><br>
                                <li>Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), ele não deve ter preço e será necessário atrelar o código externo deste subitem, no formato Código Grupo de Observação|Código de Observação (xx|yy).</li><br>
                                <li>Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste subitem, no formato 'xx.xx.xx.xx.xx'.</li><br>
                                <p align='justify'>O código externo deve ser igual ao cadastrado previamente no Retail. Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");
    
                        $observationCode = $obs['ref'];
    
                        if(!strpos($observationCode, "|"))
                            throw new Exception\ParamsException("<p align='justify'>
                                O subitem {$obs['title']} foi cadastrado sem preço e não existe um subitem cadastrado no Retail com o código externo informado: {$observationCode}.</p><br>
                                <li>Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código externo deste subitem. O formato correto é Código Grupo de Observação|Código de Observação (xx|yy).</li><br>
                                <li>Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.</li><br>
                                <p align='justify'>Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");
    
                        $validateObs = $this->productFactory->getObservation($observationCode);
    
                        if(empty($validateObs))
                            throw new Exception\ParamsException("<p align='justify'>
                                Não existe uma observação atrelada ao código externo informado.</p><br>
                                <li>Item do pedido: {$obs['title']}.</li><br>
                                <li>Código externo: {$obs['ref']}.</li><br>
                                <p align='justify'>Verifique no Odhen Retail o código externo desta observação e faça a correção no portal Neemo. Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.</p>");
    
                        $productDetails['ITEM_OBSERVATION'] = array(
                            'CDFILIAL'        => $generalParams["CDFILIAL"],  
                            'NRVENDAREST'     => $orderData["NRVENDAREST"], 
                            'NRCOMANDA'       => $orderData["NRCOMANDA"], 
                            'NRPRODCOMVEN'    => $productDetails["NRPRODCOMVEN"],  
                            'CDGRPOCOR'       => $validateObs['CDGRPOCOR'],   
                            'CDOCORR'         => $validateObs['CDOCORR'],  
                            'NRPRODCOMVENOBS' => null,  
                            'QTPRODCOMVENOBS' => $obs['quantity']      
                        );

                        for ($i = 0 ; $i < $obs["quantity"] ; $i ++)
                            $productDetails["TXPRODCOMVEN"] .= $validateObs['DSOCORR']."; ";

                    }    
                    

                }
            }
        }

        $productReturn = array(
            "CDFILIAL"       => $generalParams["CDFILIAL"],
            "NRVENDAREST"    => $orderData["NRVENDAREST"],
            "NRCOMANDA"      => $orderData["NRCOMANDA"],
            "NRPRODCOMVEN"   => $productDetails["NRPRODCOMVEN"],
            "CDPRODUTO"      => $productDetails["CDPRODUTO"],
            "QTPRODCOMVEN"   => $productDetails["quantity"],
            "VRPRECCOMVEN"   => $productDetails["price"],  
            "TXPRODCOMVEN"   => $productDetails["TXPRODCOMVEN"],
            "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
            "VRDESCCOMVEN"   => isset($productDetails["VRDESCCOMVEN"]) ? $productDetails["VRDESCCOMVEN"] : 0,
            "NRLUGARMESA"    => "01",
            "IDPRODIMPFIS"   => "N",
            "CDLOJA"         => $generalParams["CDLOJA"],
            "VRACRCOMVEN"    => 0,
            "NRSEQPRODCOM"   => null,
            "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
            "VRPRECCLCOMVEN" => 0,
            "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
            "CDPRODPROMOCAO" => null,
            "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
            "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $orderData["cpf_in_note"]),
            "DSOBSPEDDIGCMD" => $product["notes"],
        );

        if(isset($productDetails['ITEM_OBSERVATION'])){
            $productReturn["ITEM_OBSERVATION"] = $productDetails['ITEM_OBSERVATION'];
        }

        
        return array_merge($product, $productReturn);
    }
    
    private function processProductAsComboWithExtCode($generalParams, $product, $order, &$nrSeqProdCom, &$observations) {
        $productItens = array();
        
        for ($i = 0 ; $i < $product["quantity"] ; $i++) {
        
            $productNrSeqProdCom = str_pad((string) ($nrSeqProdCom + 1), 6, '0', STR_PAD_LEFT);
            $productNrSeqProdComPai = $productNrSeqProdCom;
            $nrSeqProdCom++;

            $productDetails = $this->productFactory->getProduct($product["variacao_ref"]);
            if (empty($productDetails))
                throw new Exception\ParamsException("<p align='justify'>
                    Não existe um produto atrelado ao código externo informado.</p><br>
                    <li>Produto: {$product['title']}.</li><br>
                    <li>Código externo: {$product['variacao_ref']}.</li><br>
                    <p align='justify'>Verifique no Odhen Retail o código externo deste produto e faça a correção no portal Neemo.</p>");
                    
            $productDetails = array_merge($productDetails, $product);
    

            $productDetails["NRPRODCOMVEN"] = $productNrSeqProdCom;
            $productDetails["TXPRODCOMVEN"] = " ";
            $productDetails["TXPRODCOMVEN"] .= (isset($product['notes']) && !empty($product['notes'])) ? $product['notes'] . "; " : "";
            
            $itemReturn = array(
                "CDFILIAL"          => $generalParams["CDFILIAL"],
                "NRVENDAREST"       => $order["NRVENDAREST"],
                "NRCOMANDA"         => $order["NRCOMANDA"],
                "NRPRODCOMVEN"      => $productDetails["NRPRODCOMVEN"],
                "CDPRODUTO"         => $productDetails["CDPRODUTO"],
                "QTPRODCOMVEN"      => 1,
                "VRPRECCOMVEN"      => $productDetails["price"],  
                "TXPRODCOMVEN"      => $productDetails["TXPRODCOMVEN"],
                "IDSTPRCOMVEN"      => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                "VRDESCCOMVEN"      => 0,
                "NRLUGARMESA"       => "01",
                "IDPRODIMPFIS"      => "N",
                "CDLOJA"            => $generalParams["CDLOJA"],
                "VRACRCOMVEN"       => 0,
                "NRSEQPRODCOM"      => $this->formatNrSeqProdCom($productNrSeqProdComPai),
                "NRSEQPRODCUP"      => $generalParams["NRSEQPRODCUP"],
                "VRPRECCLCOMVEN"    => 0,
                "CDCAIXACOLETOR"    => $generalParams["CDCAIXA"],
                "CDPRODPROMOCAO"    => $productDetails["CDPRODUTO"],
                "CDVENDEDOR"        => $generalParams["CDVENDEDOR"],
                "NRINSCRCONS"       => preg_replace("/[^0-9]/", "", $order["cpf_in_note"]),
                "DSOBSPEDDIGCMD"    => null,
                "ITEM_OBSERVATION"  => []
            );
            
            $CDPRODPROMO = $productDetails["CDPRODUTO"];

            array_push($productItens, $itemReturn);

            $lastSubItemWithPrice = null;

            foreach ($product["ComplementCategories"] as $complementCategories) {
            
                foreach ($complementCategories["Complements"] as $item) {
                    
                    //========================== Observações Sem Preço ==========================
                    if(isset($item['price_un']) && !$item['price_un'] && !empty($item['title'])) {
                        // Observações tipo O e R
                        
                        if(!isset($item['ref']) || empty($item['ref']))
                            throw new Exception\ParamsException("<p align='justify'>
                                O subitem {$item['title']} foi cadastrado sem preço e sem código externo. Apesar do preço não ser uma exigência (ver abaixo), o código externo é obrigatório para todos os subitems.</p><br>
                                <li>Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), ele não deve ter preço e será necessário atrelar o código externo deste subitem, no formato Código Grupo de Observação|Código de Observação (xx|yy).</li><br>
                                <li>Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário atrelar um valor igual ou superior a R$ 0,01 e atrelar o código externo deste subitem, no formato 'xx.xx.xx.xx.xx'.</li><br>
                                <p align='justify'>O código externo deve ser igual ao cadastrado previamente no Retail. Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");
    
                        $observationCode = $item['ref'];
    
                        if(!strpos($observationCode, "|"))
                            throw new Exception\ParamsException("<p align='justify'>
                                O subitem {$item['title']} foi cadastrado sem preço e não existe um subitem cadastrado no Retail com o código externo informado: {$observationCode}.</p><br>
                                <li>Caso o subitem seja uma observação (por exemplo, 'Sem salada' ou 'Ao ponto'), será necessário corrigir o código externo deste subitem. O formato correto é Código Grupo de Observação|Código de Observação (xx|yy).</li><br>
                                <li>Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), será necessário corrigir o valor do subitem, que deve ser igual ou superior a R$ 0,01.</li><br>
                                <p align='justify'>Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");
    
                        $validateObs = $this->productFactory->getObservation($observationCode);
    
                        if(empty($validateObs))
                            throw new Exception\ParamsException("<p align='justify'>
                                Não existe uma observação atrelada ao código externo informado.</p><br>
                                <li>Item do pedido: {$item['title']}.</li><br>
                                <li>Código externo: {$item['ref']}.</li><br>
                                <p align='justify'>Verifique no Odhen Retail o código externo desta observação e faça a correção no portal Neemo. Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da integração.</p>");
    
                        array_push($productItens[0]['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $generalParams["CDFILIAL"],  
                            'NRVENDAREST'     => $order["NRVENDAREST"], 
                            'NRCOMANDA'       => $order["NRCOMANDA"], 
                            'NRPRODCOMVEN'    => $productDetails["NRPRODCOMVEN"],  
                            'CDGRPOCOR'       => $validateObs['CDGRPOCOR'],   
                            'CDOCORR'         => $validateObs['CDOCORR'],  
                            'NRPRODCOMVENOBS' => null,  
                            'QTPRODCOMVENOBS' => $item['quantity']      
                        ]);
                        //$this->processObservationWithoutPrice($productDetails["NRPRODCOMVEN"], $validateObs['DSOCORR'], $item['quantity'], $productItens); --Depreciado
                        $this->processObservationWithoutPrice($lastSubItemWithPrice, $validateObs['DSOCORR'], $item['quantity'], $productItens);

                        continue;                    
                    }
                    
                    //======================= Fim de Observações Sem Preço ======================

                    $itemDetails = $this->productFactory->getProduct($item["ref"]);
                    
                    if (empty($itemDetails)){
                        if(strpos($item['ref'], "|")){
                            throw new Exception\ParamsException("<p align='justify'>
                                O subitem '{$item['title']}' possui preço e o código externo ({$item['ref']}) está no formato de uma observação que não possui preço (XX|YY).</p><br>
                                <li>Caso o subitem seja uma observação ou remoção (por exemplo, 'Sem salada' ou 'Ao ponto'), você deve remover o preço deste subitem.</li><br>
                                <li>Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), você deve corrigir o código externo deste subitem, no formato 'xx.xx.xx.xx.xx'.</li><br>
                                <p align='justify'>Verifique no Retail o código externo e o valor desta observação e faça a correção no portal Neemo. Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");
                        } else {
                            throw new Exception\ParamsException("<p align='justify'>
                                O subitem '{$item['title']}' foi cadastrado sem código externo. Verifique no Odhen Retail o código externo deste produto e faça a correção no portal da Neemo. Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");
                        }
                    }
                        
                    $itemDetails = array_merge($itemDetails, $item);
                    
                    for($y = 0; $y < $itemDetails["quantity"]; $y++) {
                        $productNrSeqProdCom = str_pad((string) ($nrSeqProdCom + 1), 6, '0', STR_PAD_LEFT);
                        $nrSeqProdCom++;
                        
                        $itemDetails["NRPRODCOMVEN"] = $productNrSeqProdCom;
                        $itemDetails["TXPRODCOMVEN"] = $productDetails["TXPRODCOMVEN"];
                        $itemDetails["TXPRODCOMVEN"] .= (isset($item['notes']) && !empty($item['notes'])) ? $item['notes'] . "; " : "";
                        //$itemDetails["TXPRODCOMVEN"] = (isset($product['notes']) && !empty($product['notes'])) ? $product['notes'] : "";
                        
                        $itemReturn = array(
                            "CDFILIAL"       => $generalParams["CDFILIAL"],
                            "NRVENDAREST"    => $order["NRVENDAREST"],
                            "NRCOMANDA"      => $order["NRCOMANDA"],
                            "NRPRODCOMVEN"   => $itemDetails["NRPRODCOMVEN"],
                            "CDPRODUTO"      => $itemDetails["CDPRODUTO"],
                            "QTPRODCOMVEN"   => 1,
                            "VRPRECCOMVEN"   => $itemDetails["price_un"],  
                            "TXPRODCOMVEN"   => $itemDetails["TXPRODCOMVEN"],
                            "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                            "VRDESCCOMVEN"   => 0,
                            "NRLUGARMESA"    => "01",
                            "IDPRODIMPFIS"   => "N",
                            "CDLOJA"         => $generalParams["CDLOJA"],
                            "VRACRCOMVEN"    => 0,
                            "NRSEQPRODCOM"   => $this->formatNrSeqProdCom($productNrSeqProdComPai),
                            "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                            "VRPRECCLCOMVEN" => 0,
                            "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                            "CDPRODPROMOCAO" => $CDPRODPROMO,
                            "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                            "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $order["cpf_in_note"]),
                            "DSOBSPEDDIGCMD" => null
                        );
                        
                        $lastSubItemWithPrice = $itemReturn['NRPRODCOMVEN'];
                        array_push($productItens, $itemReturn);   
                    }

                }
            }
        }
        
        return $productItens;
    }
        
    private function processProductAsComboWithoutExtCode($generalParams, $product, $orderData, &$nrSeqProdCom) {

        $productItens = array();

        for ($i = 0 ; $i < $product["quantity"] ; $i++) {

            foreach ($product["ComplementCategories"] as $complement) {
                
                if(!count($complement['Complements'])) {
                    continue;
                }
                
                foreach($complement['Complements'] as $item) {
                    
                    if(empty($item["ref"])){
                        throw new Exception\ParamsException("<p align='justify'>
                            O produto {$item['title']} foi cadastrado sem código externo. Verifique no Odhen Retail o código externo deste produto e faça a correção no portal da Neemo. Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");
                    }
                    
                    if(empty($item["price_un"])){
                        throw new Exception\ParamsException("<p align='justify'>
                            O subitem '{$item['title']}' foi cadastrado sem preço. Entretanto, este subitem está relacionado a um agrupador sem preço e sem código externo. Nesse caso, todos os subitens devem possuir preço igual ou superior a R$ 0,01 e código externo no formato 'xx.xx.xx.xx.xx'. Verifique no Retail o código externo e o valor desta observação e faça a correção no portal Neemo. Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");
                    }
                    
                    $itemDetails = $this->productFactory->getProduct($item["ref"]);
                    
                    if (empty($itemDetails))
                        throw new Exception\ParamsException("<p align='justify'>
                            O subitem '{$item['title']}', código externo ({$item['ref']}) não foi encontrado no sistema. Verifique no Retail o código externo e o valor desta observação e faça a correção no portal Neemo. Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");

                            // O subitem '{$item['title']}' possui preço e o código externo ({$item['ref']}) está no formato de uma observação que não possui preço (XX|YY).</p><br>
                            // <li>Caso o subitem seja uma observação ou remoção (por exemplo, 'Sem salada' ou 'Ao ponto'), você deve remover o preço deste subitem.</li><br>
                            // <li>Caso o subitem seja um acréscimo (por exemplo, 'Dobro de Queijo'), você deve corrigir o código externo deste subitem, no formato 'xx.xx.xx.xx.xx'.</li><br>
                            // <p align='justify'>Verifique no Retail o código externo e o valor desta observação e faça a correção no portal Neemo. Para mais informações, consulte a seção <b>3. Parametrização do Cardápio</b>, no manual da Neemo.</p>");

                    $itemDetails = array_merge($itemDetails, $item);
                    
                    for($y = 0; $y < $itemDetails["quantity"]; $y++) {
                        
                        $productNrSeqProdCom = str_pad((string) ($nrSeqProdCom + 1), 6, '0', STR_PAD_LEFT);
                        $nrSeqProdCom++;
    
                        $itemDetails["NRPRODCOMVEN"] = $productNrSeqProdCom;
                        $itemDetails["TXPRODCOMVEN"] = " ";
                        $itemDetails["TXPRODCOMVEN"] .= (isset($product['notes']) && !empty($product['notes'])) ? $product['notes'] . "; " : "";
                        //$itemDetails["TXPRODCOMVEN"] = (isset($product['notes']) && !empty($product['notes'])) ? $product['notes'] : "";
                        
                        $itemReturn = array(
                            "CDFILIAL"       => $generalParams["CDFILIAL"],
                            "NRVENDAREST"    => $orderData["NRVENDAREST"],
                            "NRCOMANDA"      => $orderData["NRCOMANDA"],
                            "NRPRODCOMVEN"   => $itemDetails["NRPRODCOMVEN"],
                            "CDPRODUTO"      => $itemDetails["CDPRODUTO"],
                            "QTPRODCOMVEN"   => 1,
                            "VRPRECCOMVEN"   => $itemDetails["price_un"],  
                            "TXPRODCOMVEN"   => $itemDetails["TXPRODCOMVEN"],
                            "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                            "VRDESCCOMVEN"   => 0,
                            "NRLUGARMESA"    => "01",
                            "IDPRODIMPFIS"   => "N",
                            "CDLOJA"         => $generalParams["CDLOJA"],
                            "VRACRCOMVEN"    => 0,
                            "NRSEQPRODCOM"   => null,
                            "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                            "VRPRECCLCOMVEN" => 0,
                            "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                            "CDPRODPROMOCAO" => null,
                            "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                            "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $orderData["cpf_in_note"]),
                            "DSOBSPEDDIGCMD" => null
                        );
                        
                        array_push($productItens, $itemReturn);
                    }
                }
            }
        }
        
        return $productItens;
    }
    
    public function insertProduct ($product, $generalParams, $orderData) {
        $this->productFactory->insertProduct($product, $generalParams, $orderData);
        $this->processObservations($product, $generalParams, $orderData);
    }
    
    private function processObservations ($productDetails, $generalParams, $orderData) {
        if (isset($productDetails["ComplementCategories"]) && count($productDetails["ComplementCategories"]) > 0) {
            foreach ($productDetails["ComplementCategories"] as $categorie)
                $this->insertObservation($categorie["Complements"], $productDetails, $generalParams, $orderData);
        }
    }
    
    private function getObservationName ($observation, $generalParams) {
        if (empty($observation["variacao_ref"])) {
            if ($observation["ref"] === "")
                throw new Exception\ParamsException("A observação: ".$observation["title"].", não possui código de integração parametrizado.", 109);
            
            $observation["ref"] = $observation["ref"];
        } else $observation["ref"] = $observation["variacao_ref"];
        
        $observationDetails = $this->productFactory->getObservation($observation["ref"], $generalParams);
        
        if (empty($observationDetails))
            throw new Exception\ParamsException("Não foi encontrada ocorrência para o código de integração: ".$observation["ref"], 110);
        
        return $observationDetails["DSOCORR"];
    }
    
    private function insertObservation ($observations, $productDetails, $generalParams, $orderData) {
        foreach ($observations as $observation) {
            if (empty($observation["variacao_ref"])) {
                if ($observation["ref"] === "")
                    throw new Exception\ParamsException("A observação: ".$observation["title"].", não possui código de integração parametrizado.", 109);
                
                $observation["ref"] = $observation["ref"];
            } else $observation["ref"] = $observation["variacao_ref"];
            
            // Pular iteracao de produto que nao é observacao
            if(strpos($observation['ref'], "@") === false) {
                continue;
            }
            
            $observationDetails = $this->productFactory->getObservation($observation["ref"], $generalParams);
            
            if (empty($observationDetails))
                throw new Exception\ParamsException("Não foi encontrada ocorrência para o código de integração: ".$observation["ref"]." - Produto: ".$observation["title"], 110);
            
            $observationDetails = array_merge($observationDetails, $observation);
            $observationDetails["NRPRODCOMVEN"] = $this->util->newCode("ITCOMANDAVEN".$generalParams["CDFILIAL"].$orderData["NRCOMANDA"], 6);
            
            if ($observationDetails["price_un"] != 0) {
                $product = array(
                    "CDFILIAL"       => $generalParams["CDFILIAL"],
                    "NRVENDAREST"    => $orderData["NRVENDAREST"],
                    "NRCOMANDA"      => $orderData["NRCOMANDA"],
                    "NRPRODCOMVEN"   => $observationDetails["NRPRODCOMVEN"],
                    "CDPRODUTO"      => $observationDetails["CDPRODUTO"],
                    "QTPRODCOMVEN"   => $observationDetails["quantity"] * $productDetails["quantity"],
                    "VRPRECCOMVEN"   => $observationDetails["price_un"],
                    "TXPRODCOMVEN"   => null,
                    "IDSTPRCOMVEN"   => $generalParams["IDCONTROPROD"] === "S" ? "1" : "4",
                    "VRDESCCOMVEN"   => 0,
                    "NRLUGARMESA"    => "01",
                    "IDPRODIMPFIS"   => "N",
                    "CDLOJA"         => $generalParams["CDLOJA"],
                    "VRACRCOMVEN"    => 0,
                    "NRSEQPRODCOM"   => $productDetails["NRSEQPRODCOM"],
                    "NRSEQPRODCUP"   => $generalParams["NRSEQPRODCUP"],
                    "VRPRECCLCOMVEN" => 0,
                    "CDCAIXACOLETOR" => $generalParams["CDCAIXA"],
                    "CDPRODPROMOCAO" => null,
                    "CDVENDEDOR"     => $generalParams["CDVENDEDOR"],
                    "NRINSCRCONS"    => preg_replace("/[^0-9]/", "", $orderData["cpf_in_note"]),
                    "DSOBSPEDDIGCMD" => ""
                );
                
                $this->productFactory->insertProduct($product, $generalParams, $orderData);
            }
            
            $observationParams = array(
                "CDFILIAL"        => $generalParams["CDFILIAL"],
                "NRVENDAREST"     => $orderData["NRVENDAREST"],
                "NRCOMANDA"       => $orderData["NRCOMANDA"],
                "NRPRODCOMVEN"    => $productDetails["NRPRODCOMVEN"],
                "NRPRODCOMVENOBS" => $observationDetails["price_un"] != 0 ? $observationDetails["NRPRODCOMVEN"] : null,
                "CDOCORR"         => $observationDetails["CDOCORR"],
                "CDGRPOCOR"       => $observationDetails["CDGRPOCOR"],
                "QTPRODCOMVENOBS" => $observationDetails["quantity"] * $productDetails["quantity"]
            );
            
            $this->productFactory->insertObservation($observationParams);
        }
    }
    
    public function shareDiscount($initialDiscount, &$products) {

        $idxToApplyDiscount   = [];
        $totalDiscount        = $initialDiscount;
        $discountLeft         = $initialDiscount;
        $productsTotalValue   = "0.00";
        $discountGivenShare   = "0.00";
        $maxValueItem         = "0.00";
        $maxValueIdx          = -1;
        
        foreach($products as &$product) {
            $product['VRDESCCOMVEN']  = 0;
            $product['discountShare'] = "0.00";
        }
        
        if(isset($product))
            unset($product);

        foreach($products as $i => $product) {
            if(bccomp($product["VRPRECCOMVEN"], "0.01", 2) == 1) {
                $productsTotalValue = bcadd($productsTotalValue, $product["VRPRECCOMVEN"], 2);
                array_push($idxToApplyDiscount, $i);
            }
        }

        foreach($idxToApplyDiscount as $idxWithDiscount) {
            $discountShare = bcdiv((bcmul($products[$idxWithDiscount]["VRPRECCOMVEN"], "100.00", 2)), $productsTotalValue, 2);
            $products[$idxWithDiscount]["discountShare"] = $discountShare;

            $discountGivenShare = bcadd($discountGivenShare, $discountShare, 2);
        }

        foreach($products as $i => $product) {
            if(bccomp($product['discountShare'], $maxValueItem, 2) == 1) {
                $maxValueIdx  = $i;
                $maxValueItem = $product['discountShare'];
            }            
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["discountShare"] = bcadd($products[$maxValueIdx]["discountShare"], bcsub("100.00", $discountGivenShare, 2), 2);
        }
    
        foreach($idxToApplyDiscount as $idxWithDiscount) {
        
            $itemDiscount = bcdiv((bcmul($products[$idxWithDiscount]["discountShare"], $totalDiscount, 2)), "100.00", 2);
            $products[$idxWithDiscount]["VRDESCCOMVEN"] = $itemDiscount;

            $discountLeft = bcsub($discountLeft, $itemDiscount, 2);
        }

        if($maxValueIdx != -1) {
            $products[$maxValueIdx]["VRDESCCOMVEN"] = bcadd($products[$maxValueIdx]["VRDESCCOMVEN"], $discountLeft, 2);
            $discountLeft = "0.00";
        }
        
        return $products;
    }
    
    private function processObservationWithoutPrice($nrProdComVenKey, $observation, $quantity, &$products) {
        foreach($products as &$product) {
            if($nrProdComVenKey != $product['NRPRODCOMVEN']) {
                continue;
            }

            for($i = 0; $i < $quantity; $i++) {
                if(empty($product["TXPRODCOMVEN"])) {
                    $product["TXPRODCOMVEN"] = " $observation; ";
                } else {
                    $product["TXPRODCOMVEN"] .= "$observation; ";
                }
            }            
        }
        
        if(isset($product)) {
            unset($product);
        }
    }
    
    private function formatNrSeqProdCom($seq) {
        return str_pad((string) ((int) $seq), 3, '0', STR_PAD_LEFT);
    }
}